use std::{collections::HashMap, fs};

use log::error;

// ENUMS

/// Specifies a limit type, whether block or silence
#[derive(Clone, Copy, Debug, Eq, PartialEq)]
pub enum Limit {
    Block,
    Silence,
}

// STRUCTS

/// Wraps a host's weighted limits
#[derive(Debug, Default, Eq, PartialEq)]
pub struct LimitIndices {
    pub block: u16,
    pub silence: u16,
}

impl LimitIndices {
    pub fn add_limit(&mut self, limit: Limit, weight: u16) -> &mut Self {
        match limit {
            Limit::Block => self.block += weight,
            Limit::Silence => self.silence += weight,
        }
        self
    }
}

impl From<(u16, u16)> for LimitIndices {
    /// Creates indices from a tuple of two `u16` weights. Useful mostly for
    /// testing.
    fn from(value: (u16, u16)) -> Self {
        Self {
            block: value.0,
            silence: value.1,
        }
    }
}

/// Contains a mapping of hosts to limits as well as a trust value which is
/// used to weight limits when building a merged list
#[derive(Debug, PartialEq, Eq)]
pub struct LimitList {
    pub limits: HashMap<String, Limit>,
    pub trust: u16,
}

impl Default for LimitList {
    fn default() -> Self {
        Self {
            limits: HashMap::new(),
            trust: 100,
        }
    }
}

impl From<HashMap<String, Limit>> for LimitList {
    fn from(map: HashMap<String, Limit>) -> Self {
        Self {
            limits: map,
            trust: 100,
        }
    }
}

impl LimitList {
    fn add_host(&mut self, host: &str, limit: Limit) -> &mut Self {
        self.limits.insert(host.to_string(), limit);
        self
    }

    pub fn build(map: HashMap<String, Limit>, trust: u16) -> Self {
        let mut src = Self::from(map);
        src.trust = trust;
        src
    }

    pub fn import_file(&mut self, path: &str, limit: Limit) -> &mut Self {
        let contents = fs::read_to_string(path).unwrap();
        for host in contents.lines().filter(|line| !line.is_empty()) {
            self.add_host(host, limit);
        }

        self
    }
}

/// A map of hosts (as strings) to their limit weights
#[derive(Debug, Default, Eq, PartialEq)]
pub struct MergedLimitList {
    pub map: HashMap<String, LimitIndices>,
    pub max: u16,
}

impl MergedLimitList {
    pub fn add_limit_list(&mut self, src: LimitList) -> &mut Self {
        for (host, limit) in src.limits.into_iter() {
            let entry = self.map.entry(host).or_default();
            entry.add_limit(limit, src.trust);
        }

        self.max += src.trust;
        self
    }

    pub fn export_file(
        self,
        block_path: &str,
        mute_path: &str,
        indices: (u16, u16),
    ) -> std::io::Result<()> {
        if self.map.is_empty() {
            error!("Nothing to export!");
            return Ok(());
        }

        let (block_thresh, mute_thresh) = indices;
        let mut block_output: String = String::default();
        let mut mute_output: String = String::default();

        for item in self.map.into_iter() {
            let (block_trust, mute_trust) = (item.1.block, item.1.silence);

            if block_trust >= block_thresh {
                block_output.push_str(&(item.0.clone() + "\n"));
            }

            if mute_trust >= mute_thresh {
                mute_output.push_str(&(item.0.clone() + "\n"));
            }
        }

        fs::write(block_path, alphabetize_lines(block_output))?;
        fs::write(mute_path, alphabetize_lines(mute_output))?;

        Ok(())
    }
}

pub fn alphabetize_lines(string: String) -> String {
    let mut v: Vec<&str> = string.lines().collect();
    v.sort_by_key(|a| a.to_lowercase());

    v.join("\n") + "\n"
}
